// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_continuous_deployment/src/core/commands/build_app/build_app_task.dart';
import 'package:wyatt_continuous_deployment/src/core/constants/app_constants.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/decrypt_keys_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/config/get_android_config_usecase.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/decrypt_keys/decrypt_keys_usecase.dart';

part 'build_app_bundle_state.dart';

class BuildAppBundleTask extends BuildAppTask {
  factory BuildAppBundleTask() => BuildAppBundleTask._(
        flutter: getIt(),
        decryptKeysUsecase: getIt(),
        getAndroidConfigUsecase: getIt(),
        checkToolsUsecase: getIt(),
        checkConfigFileUsecase: getIt(),
      );

  BuildAppBundleTask._({
    required DecryptKeysUsecase decryptKeysUsecase,
    required GetAndroidConfigUsecase getAndroidConfigUsecase,
    required super.flutter,
    required super.checkToolsUsecase,
    required super.checkConfigFileUsecase,
  })  : _decryptKeysUsecase = decryptKeysUsecase,
        _getAndroidConfigUsecase = getAndroidConfigUsecase;

  final DecryptKeysUsecase _decryptKeysUsecase;
  final GetAndroidConfigUsecase _getAndroidConfigUsecase;

  @override
  FutureOr<void> buildApp(Context ctx) async {
    emit(BuildAppBundleStateGetAndroidConfig());

    // 1. Get Android config
    final getAndroidConfigResult =
        await _getAndroidConfigUsecase(ctx.configFile.path);
    if (getAndroidConfigResult.isErr) {
      return emit(
        TaskStateError(message: getAndroidConfigResult.err.toString()),
      );
    }

    // 2. Decrypt Android keys
    emit(BuildAppBundleStateDecryptAndroidKeys());
    final decryptAndroidkeysResult = await _decryptKeysUsecase(
      DecryptKeysParameters(
        encryptedFilePath: getAndroidConfigResult.ok!.encryptedKeyFile.path,
        passphrase: ctx.environment[AppConstants.androidKeysSecretPassphrase]
            .toString(),
      ),
    );
    if (decryptAndroidkeysResult.isErr) {
      return emit(
        TaskStateError(message: decryptAndroidkeysResult.err.toString()),
      );
    }

    // 3. Build Android App Bundle
    emit(BuildAppBundleStateBuildAppBundle());
    final buildAppBundleResult = await flutter
        .buildAppBundle(getAndroidConfigResult.ok!.flutterArgs.split(' '));
    if (buildAppBundleResult.isErr) {
      return emit(
        TaskStateError(message: decryptAndroidkeysResult.err.toString()),
      );
    }

    emit(const TaskStateSucceed());
  }
}
