// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/utils/logging.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/decrypt_keys_parameters.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/usecase.dart';

@usecase
class DecryptKeysUsecase extends AsyncUseCase<DecryptKeysParameters, void> {
  const DecryptKeysUsecase() : super();

  @override
  FutureOr<void> onStart(DecryptKeysParameters? params) {
    if (params == null) {
      throw const ClientException(
        'DecryptKeysUsecase: DecryptKeysParameters is null',
      );
    }
    if (params.encryptedFilePath.isEmpty) {
      throw const ClientException(
        'DecryptKeysUsecase: Encrypted file path is empty',
      );
    }
    if (!params.encryptedFilePath.endsWith('.gpg')) {
      throw const ClientException(
        'DecryptKeysUsecase: Enncrypted file is not a .gpg file',
      );
    }
  }

  @override
  FutureOrResult<void> execute(DecryptKeysParameters? params) =>
      unsafeAsync(() async {
        // Define gpg command
        final gpgFilePath =
            params!.encryptedFilePath.replaceAll(RegExp(r'\.gpg$'), '');
        final outputGpgFilePath =
            gpgFilePath.endsWith('.zip') ? gpgFilePath : '$gpgFilePath.zip';

        final gpgCommand = 'gpg --quiet --batch --yes --decrypt '
            '--passphrase=${params.passphrase} '
            '--output $outputGpgFilePath ${params.encryptedFilePath}';

        logger.detail(
          '''DecryptKeysUsecase: Decrypting the file with the command: ${gpgCommand.replaceAll(params.passphrase, '*' * params.passphrase.length)}''',
        );

        final progress = logger.progress('Decrypting keys...');

        // Verify if the command was executed successfully
        final gpgResult = await Process.run('sh', ['-c', gpgCommand]);

        logger.detail(gpgResult.stdout.toString());

        if (gpgResult.exitCode == 0) {
          // Move the extracted files to the current directory
          final folder = Directory(outputGpgFilePath.replaceAll('.zip', ''));

          progress.update('Unzipping...');

          final unzipResult = await Process.run('sh', [
            '-c',
            'unzip -o $outputGpgFilePath -d ${folder.parent.path} && mv ${folder.path}/* ${folder.parent.path}',
          ]);

          logger.detail(unzipResult.stdout.toString());

          if (unzipResult.exitCode != 0) {
            progress.fail('Error during the unzip of the file');
            throw ClientException(
              'DecryptKeysUsecase: Error during the unzip '
              'of the file : ${unzipResult.stderr}',
            );
          }

          return progress.complete('Keys decrypted!');
        } else {
          progress.fail('Error during the decryption of the GPG file');
          throw ClientException(
            'DecryptKeysUsecase: Error during the decryption '
            'of the GPG file : ${gpgResult.stderr}',
          );
        }
      });
}
