// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/core/model_mapper.dart';
import 'package:wyatt_crud_bloc/src/data/data_sources/crud_data_source.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';
import 'package:wyatt_crud_bloc/src/domain/repositories/crud_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

/// {@template crud_repository_impl}
/// A repository that implements the [CrudRepository] interface.
/// {@endtemplate}
class CrudRepositoryImpl<Model> extends CrudRepository<Model> {
  /// {@macro crud_repository_impl}
  const CrudRepositoryImpl({
    required this.modelMapper,
    required CrudDataSource crudDataSource,
  }) : _crudDataSource = crudDataSource;

  final CrudDataSource _crudDataSource;

  @override
  final ModelMapper<Model> modelMapper;

  @override
  FutureOrResult<void> create(Model object, {String? id}) =>
      Result.tryCatchAsync<void, AppException, AppException>(
        () async {
          await _crudDataSource.create(modelMapper.toJson(object), id: id);
        },
        (error) => error,
      );

  @override
  FutureOrResult<Model?> get(String id) =>
      Result.tryCatchAsync<Model?, AppException, AppException>(
        () async => modelMapper.fromJson(await _crudDataSource.get(id)),
        (error) => error,
      );

  @override
  FutureOrResult<List<Model?>> getAll() =>
      Result.tryCatchAsync<List<Model?>, AppException, AppException>(
        () async {
          final lst = await _crudDataSource.getAll();
          if (lst.isNotNull) {
            return lst!.map((raw) {
              try {
                return modelMapper.fromJson(raw as Map<String, dynamic>?);
              } catch (e) {
                return null;
              }
            }).toList();
          }
          throw const ServerException();
        },
        (error) => error,
      );

  @override
  FutureOrResult<void> update(
    String id, {
    Model? object,
    Map<String, dynamic>? raw,
  }) =>
      Result.tryCatchAsync<void, AppException, AppException>(
        () async {
          if (object != null) {
            await _crudDataSource.update(
              id,
              object: modelMapper.toJson(object),
            );
          } else if (raw != null) {
            await _crudDataSource.update(id, object: raw);
          } else {
            throw const ServerException();
          }
        },
        (error) => error,
      );

  @override
  FutureOrResult<void> updateAll(Map<String, Object?> raw) =>
      Result.tryCatchAsync<void, AppException, AppException>(
        () async => _crudDataSource.updateAll(raw),
        (error) => error,
      );

  @override
  FutureOrResult<void> delete(String id) =>
      Result.tryCatchAsync<void, AppException, AppException>(
        () async => _crudDataSource.delete(id),
        (error) => error,
      );

  @override
  FutureOrResult<void> deleteAll() =>
      Result.tryCatchAsync<void, AppException, AppException>(
        () async => _crudDataSource.deleteAll(),
        (error) => error,
      );

  @override
  FutureOrResult<List<Model?>> search(List<Query> conditions) =>
      Result.tryCatchAsync<List<Model?>, AppException, AppException>(
        () async {
          final lst = await _crudDataSource.search(conditions);
          if (lst.isNotNull) {
            return lst!.map((raw) {
              try {
                return modelMapper.fromJson(raw as Map<String, dynamic>?);
              } catch (e) {
                return null;
              }
            }).toList();
          }
          throw const ServerException();
        },
        (error) => error,
      );

  @override
  StreamResult<List<Model?>> stream({
    String? id,
    List<Query>? conditions,
  }) =>
      _crudDataSource.stream(id: id, conditions: conditions).map((lst) {
        if (lst.isNotNull) {
          return Ok<List<Model?>, AppException>(
            lst!.map((raw) {
              try {
                return modelMapper.fromJson(raw as Map<String, dynamic>?);
              } catch (e) {
                return null;
              }
            }).toList(),
          );
        }
        return Err<List<Model?>, AppException>(const ServerException());
      });
}
